import 'dart:io';

import 'package:moor_ffi/moor_ffi.dart';
import 'package:moor/moor.dart';
import 'package:ocg_chat/ocg_chat.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:path_provider/path_provider.dart';
import 'package:path/path.dart' as p;

part 'moor.g.dart';

class DBMessages extends Table {
  TextColumn get id => text()();
  TextColumn get chatId => text()();
  TextColumn get content => text()();
  TextColumn get senderId => text()();
  TextColumn get type => text()();
  TextColumn get receiveData => text().nullable()();
  IntColumn get state => integer().nullable()();
  DateTimeColumn get recordedAt => dateTime()();

  @override
  Set<Column> get primaryKey => {id};
}

LazyDatabase _openConnection() {
  // the LazyDatabase util lets us find the right location for the file async.
  return LazyDatabase(() async {
    // put the database file, called db.sqlite here, into the documents folder
    // for your app.
    final dbFolder = await getApplicationDocumentsDirectory();
    final file = File(p.join(dbFolder.path, 'messagedb.sqlite'));
    return VmDatabase(file);
  });
}

@UseMoor(tables: [DBMessages])
class MessageMoorDatabase extends _$MessageMoorDatabase with MessageDatabase {
  MessageMoorDatabase() : super(_openConnection());

  int get schemaVersion => 1;

  Future<void> insertMessages(List<DBMessage> messages) async {
    await batch((batch) {
      batch.insertAll(
          dBMessages,
          messages
              .map((e) => DBMessagesCompanion.insert(
                  id: e.id,
                  chatId: e.chatId,
                  content: e.content,
                  senderId: e.senderId,
                  type: e.type,
                  receiveData: Value(e.receiveData),
                  state: Value(e.state),
                  recordedAt: e.recordedAt))
              .toList());
    });
  }

  Future<void> updateState(String messageId, int state) async {
    await (update(dBMessages)..where((tbl) => tbl.id.equals(messageId)))
        .write(DBMessagesCompanion(state: Value<int>(state)));
  }

  Future<void> updateReceiveData(String messageId, String receiveData) async {
    await (update(dBMessages)..where((tbl) => tbl.id.equals(messageId)))
        .write(DBMessagesCompanion(receiveData: Value<String>(receiveData)));
  }

  Future<AppResponse<bool>> updateMessage(Message oldMessage) async {
    try {
      DBMessage message = ModelHelper.convertToDBMessage(oldMessage);
      var rows = await (select(dBMessages)
            ..where((tbl) => tbl.id.equals(message.id)))
          .get();
      if (rows.length == 0) {
        await into(dBMessages).insert(message, mode: InsertMode.insertOrIgnore);
      } else {}
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  Stream<List<DBMessage>> subscribe() {
    return (select(dBMessages)).watch();
  }

  Future<List<DBMessage>> fetchMessages() {
    return (select(dBMessages)).get();
  }

  Future<DBMessage> fetchMessage(String id) {
    return (select(dBMessages)..where((tbl) => tbl.id.equals(id))).getSingle();
  }

  @override
  Future<AppResponse<String>> create(Message message) async {
    try {
      DBMessage dbMessage = ModelHelper.convertToDBMessage(message);
      var response = await into(dBMessages).insert(DBMessagesCompanion.insert(
          id: message.id,
          chatId: message.chatId,
          content: dbMessage.content,
          senderId: dbMessage.senderId,
          type: dbMessage.type,
          recordedAt: dbMessage.recordedAt));
      return AppResponse.named(data: "$response");
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<List<Message>>> getList({int page = 1, int count = 10}) {
    // TODO: implement getList
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<Message>> read(String id) {
    // TODO: implement read
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<bool>> deleteMessage(String id) {
    // TODO: implement deleteMessage
    throw UnimplementedError();
  }
}
