import 'package:json_annotation/json_annotation.dart';
import 'package:ocg_chat/ocg_chat.dart';
import 'package:ulid/ulid.dart';
import 'package:ocg_user/ocg_user.dart';

import 'message.dart';

part 'chat.g.dart';

enum ChatType { Personal, Group, Broadcast }

@JsonSerializable(explicitToJson: true)
class Chat {
  @JsonKey(name: "chat_id")
  final String id;
  final List<String> users;
  @JsonKey(name: "is_group", defaultValue: false)
  final bool isGroup;
  @JsonKey(defaultValue: [])
  final List<String> admins;
  @JsonKey(ignore: true)
  final ChatType type;
  @JsonKey(
    name: "group_name",
  )
  final String groupName;
  final String groupPic;
  @JsonKey(name: "unread_msg_count")
  final int unReadMsgCount;
  @JsonKey(defaultValue: false)
  final bool muted;
  @JsonKey(name: "last_read_msg_id")
  final String lastMsgId;
  @JsonKey(name: "last_msg")
  final Message lastMsg;
  @JsonKey(ignore: true)
  final User friend;
  final Map<String, dynamic> metadata;
  final bool isPublic;

  Chat(
      this.id,
      this.users,
      this.isGroup,
      this.admins,
      this.groupName,
      this.muted,
      this.lastMsgId,
      this.lastMsg,
      this.groupPic,
      this.unReadMsgCount,
      this.metadata,
      this.isPublic)
      : friend = null,
        type = ChatType.Personal;

  Chat.named(
      {this.id,
      this.users,
      this.isGroup,
      this.admins,
      this.groupName,
      this.groupPic,
      this.muted,
      this.lastMsgId,
      this.lastMsg,
      this.friend,
      this.unReadMsgCount,
      this.metadata,
      this.type,
      this.isPublic});

  Map<String, dynamic> toJson() {
    var json = _$ChatToJson(this);
    json["name"] = groupName;
    json["with"] = users;
    if (type == ChatType.Group) {
      json["is_group"] = true;
    } else if (type == ChatType.Broadcast) {
      json["is_broadcast"] = true;
    } else if (type == ChatType.Personal) {
      json["is_personal"] = true;
    }
    return json;
  }

  factory Chat.fromJson(json) {
    Chat chat = _$ChatFromJson(json);
    if (json["is_broadcast"] ?? false) {
      chat = chat.copyWith(type: ChatType.Broadcast);
    } else if (json["is_group"] ?? false) {
      chat = chat.copyWith(type: ChatType.Group);
    } else {
      chat = chat.copyWith(type: ChatType.Personal);
    }
    return chat;
  }

  String get title => ((isGroup ?? false) ? groupName : friend?.name);

  DateTime get recordedAt => lastMsg?.recordedAt;

  Chat copyWith(
      {String id,
      List<String> users,
      bool isGroup,
      List<String> admins,
      String groupName,
      bool muted,
      String lastMsgId,
      Message lastMsg,
      String groupPic,
      User friend,
      int unReadMsgCount,
      Map<String, dynamic> metadata,
      ChatType type,
      bool isPublic}) {
    return Chat.named(
        id: id ?? this.id,
        admins: admins ?? this.admins,
        groupName: groupName ?? this.groupName,
        groupPic: groupPic ?? this.groupPic,
        isGroup: isGroup ?? this.isGroup,
        lastMsg: lastMsg ?? this.lastMsg,
        lastMsgId: lastMsgId ?? this.lastMsgId,
        muted: muted ?? this.muted,
        users: users ?? this.users,
        friend: friend ?? this.friend,
        unReadMsgCount: unReadMsgCount ?? this.unReadMsgCount,
        metadata: metadata ?? this.metadata,
        type: type ?? this.type,
        isPublic: isPublic ?? this.isPublic);
  }

  DateTime get lastReadMegRecordedAt {
    try {
      return (lastMsgId != null
              ? DateTime.fromMillisecondsSinceEpoch(
                  Ulid.parse(lastMsgId).toMillis())
              : null) ??
          DateTime(1971);
    } catch (e) {
      return DateTime(1971);
    }
  }
}
